package com.codepoka.in_app_purchasing

import android.app.Activity
import android.os.Handler
import android.os.Looper
import androidx.annotation.NonNull
import com.android.billingclient.api.*

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import com.android.billingclient.api.BillingFlowParams
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding


/** InAppPurchasingPlugin */
class InAppPurchasingPlugin: FlutterPlugin, ActivityAware,MethodCallHandler, PurchasesUpdatedListener{
  private var activity: Activity? = null
  private lateinit var billingClient: BillingClient
  private lateinit var channel : MethodChannel

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    billingClient = BillingClient.newBuilder(flutterPluginBinding.applicationContext)
    .setListener(this)
    .enablePendingPurchases()
    .build()

    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "in_app_purchasing")
    channel.setMethodCallHandler(this)
    billingClient.isReady

  }
  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
    if(billingClient.connectionState == BillingClient.ConnectionState.CONNECTED) {
      billingClient.endConnection()
    }
  }
  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activity = binding.activity
  }
  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    activity = binding.activity
  }
  override fun onDetachedFromActivityForConfigChanges() {
    activity = null
  }
  override fun onDetachedFromActivity() {
    activity = null
  }


  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    when (call.method) {
      "isClientConnected" -> {
        Handler(Looper.getMainLooper()).post {
          result.success(billingClient.connectionState == BillingClient.ConnectionState.CONNECTED)
        }
      }
      "connectBillingClient" -> {
        result.success(true)
        connectBillingClient()
      }
      "disconnectBillingClient" -> {
        result.success(true)
        disconnectBillingClient()
      }
      "makePurchase" -> {
        result.success(true)
        makePurchase(call.arguments as String)
      }
      else -> {
        result.notImplemented()
      }
    }
  }

  private fun connectBillingClient() {
    if(billingClient.connectionState != BillingClient.ConnectionState.CONNECTED) {
      billingClient.startConnection(object:BillingClientStateListener{
        override fun onBillingServiceDisconnected() {}
        override fun onBillingSetupFinished(billingResult: BillingResult) {
          if(billingResult.responseCode == BillingClient.BillingResponseCode.OK){
            Handler(Looper.getMainLooper()).post {
              channel.invokeMethod("OnConnected",true)
            }
          }else{
            Handler(Looper.getMainLooper()).post {
              channel.invokeMethod("OnError","Payment failed!")
            }
          }
        }
      })
    }else{
      Handler(Looper.getMainLooper()).post {
        channel.invokeMethod("OnConnected",true)
      }
    }
  }
  private fun disconnectBillingClient(){
    billingClient.endConnection()
  }
  private fun makePurchase(pId: String){
    if(billingClient.connectionState != BillingClient.ConnectionState.CONNECTED){
      channel.invokeMethod("OnError","Service disconnected!")
      return
    }

    val productList = mutableListOf(QueryProductDetailsParams.Product.newBuilder()
        .setProductId(pId)
        .setProductType(BillingClient.ProductType.INAPP)
        .build())

    val queryProductDetailsParams = QueryProductDetailsParams.newBuilder()
    queryProductDetailsParams.setProductList(productList)

    billingClient.queryProductDetailsAsync(queryProductDetailsParams.build()) { billingResult, productDetailsList ->
      if(billingResult.responseCode == BillingClient.BillingResponseCode.OK && productDetailsList.isNotEmpty()) {

        val productParams = BillingFlowParams.ProductDetailsParams.newBuilder()
          .setProductDetails(productDetailsList[0])
          .build()

        val flowParams = BillingFlowParams.newBuilder()
          .setProductDetailsParamsList(mutableListOf(productParams))
          .build()

        if(activity != null) {
          billingClient.launchBillingFlow(activity!!, flowParams)
        }else{
          Handler(Looper.getMainLooper()).post {
            channel.invokeMethod("OnError","Payment processing failed!")
          }
        }
      }
      else {
        Handler(Looper.getMainLooper()).post {
          channel.invokeMethod("OnError","Item unavailable!")
        }
      }
    }
  }

  override fun onPurchasesUpdated(billingResult: BillingResult, purchases: MutableList<Purchase>?) {
    if (billingResult.responseCode == BillingClient.BillingResponseCode.OK && purchases != null) {
      for (purchase in purchases) {
        if (purchase.purchaseState == Purchase.PurchaseState.PURCHASED && !purchase.isAcknowledged) {
          val consumeParams = ConsumeParams.newBuilder()
            .setPurchaseToken(purchase.purchaseToken)
            .build()
          billingClient.consumeAsync(consumeParams) { billResult, token ->
            if (billResult.responseCode == BillingClient.BillingResponseCode.OK) {
              Handler(Looper.getMainLooper()).post {
                channel.invokeMethod("onPaymentSuccess",purchase.originalJson)
              }
            }else{
              Handler(Looper.getMainLooper()).post {
                channel.invokeMethod("OnError","Failed to confirm payment!")
              }
            }
          }
        }
      }
    }
    else if (billingResult.responseCode == BillingClient.BillingResponseCode.BILLING_UNAVAILABLE){
      Handler(Looper.getMainLooper()).post {
        channel.invokeMethod("OnError","Payment service unavailable!")
      }
    }
    else if (billingResult.responseCode == BillingClient.BillingResponseCode.FEATURE_NOT_SUPPORTED){
      Handler(Looper.getMainLooper()).post {
        channel.invokeMethod("OnError","Feature is not supported!")
      }
    }
    else if (billingResult.responseCode == BillingClient.BillingResponseCode.ITEM_ALREADY_OWNED){
      Handler(Looper.getMainLooper()).post {
        channel.invokeMethod("OnError","Item already purchased!")
      }
    }
    else if (billingResult.responseCode == BillingClient.BillingResponseCode.ITEM_UNAVAILABLE){
      Handler(Looper.getMainLooper()).post {
        channel.invokeMethod("OnError","Item unavailable!")
      }
    }
    else if (billingResult.responseCode == BillingClient.BillingResponseCode.SERVICE_DISCONNECTED){
      Handler(Looper.getMainLooper()).post {
        channel.invokeMethod("OnError","Service connection failed!")
      }
    }
    else if (billingResult.responseCode == BillingClient.BillingResponseCode.SERVICE_TIMEOUT){
      Handler(Looper.getMainLooper()).post {
        channel.invokeMethod("OnError"," Connection timeout!")
      }
    }
    else if (billingResult.responseCode == BillingClient.BillingResponseCode.USER_CANCELED){
      Handler(Looper.getMainLooper()).post {
        channel.invokeMethod("OnError","Payment cancelled!")
      }
    }
    else{
      Handler(Looper.getMainLooper()).post {
        channel.invokeMethod("OnError","Payment failed!")
      }
    }
  }
}
